REM -------------------------------------------------------------
REM   DXFGEN - Program to convert SDF or CDF files into DXF files
REM -------------------------------------------------------------
REM                    By Mark Lindult
REM                   copyright (c) 1988
REM                    109 Hills North
REM   Department of Landscape Architecture and Regional Planning
REM              University of Massachusetts
REM                  Amherst, MA   01003
REM -------------------------------------------------------------

DIM BLOCK$(500),ATTTAG$(500),X(500),Y(500),ATT$(500,20)
DIM LAYER$(500),NEWLAYER$(500),L(25)

REM -- PROGRAM VARIABLES ----------------------------------------
REM   E$, F$, G$ - File Names
REM   BLOCK$     - New block name for extracted objects
REM   NEWLAYER$  - Layer on which extracted objects will appear.
REM   L()        - Field length for each attribute
REM   L          - # of characters to beginning of field
REM   ATTTAG$()  - Attribute tag extracted from template file.
REM   CNT        - Counter for number of variables.
REM   TOP        - Defines the distance in inches from the
REM     centerline of the block to the location of the FIRST (or
REM     top) attribute.  Assumes 36" high text with 24" space.

REM -- SECTION ONE - Data Input ---------------------------------

INPUT "Template File Name (with .TXT): ";E$
INPUT "Name of File to be Converted  : ";F$
PRINT
PRINT "Remember the BLOCK name and new LAYER name that you assign"
PRINT "here since you will need to create a layer and block with"
PRINT "the same names when you return to AutoCAD.  The defaults "
PRINT "are:      BLOCK = `Test'  and  LAYER = `Newlayer'."
PRINT
INPUT "Block Name for extracted objects <TEST>: ";BLOCK$
INPUT "New layer Name <NEWLAYER>: ";newlayer$
INPUT "Convert Attributes? (Y/N): ";Y$
IF BLOCK$="" THEN BLOCK$="TEST"
IF NEWLAYER$="" THEN NEWLAYER$="NEWLAYER"

REM -- SECTION TWO - Input Template File ------------------------

OPEN E$ FOR INPUT AS #1
	WHILE NOT EOF(1)
		I=0:A=0
		LINE INPUT #1,A$
		IF LEFT$(A$,3)="C:Q" THEN LINE INPUT #1,A$
			J=J+1
			L(J)=VAL(MID$(A$,LEN(A$)-5,3))
		IF LEFT$(A$,3)<>"C:Q" AND LEFT$(A$,3)<>"BL:" THEN
			CNT=CNT+1                        'Count # of attributes
			WHILE A<>32
				I=I+1
				A=ASC(MID$(A$,I,1))
			WEND
			ATTTAG$(CNT)=LEFT$(A$,I-1)
		END IF
	WEND
CLOSE #1

REM -- SECTION THREE - Extract attributes -----------------------

I=0
OPEN F$ FOR INPUT AS #1
	WHILE NOT EOF(1)
		i=i+1
		INPUT #1,BLOCK$(I)
			IF LEFT$(BLOCK$(I),1)<>STR$(34) THEN GOTO SDF:
		INPUT #1,x(I),y(I),layer$(I)
			FOR J=1 TO CNT
				INPUT #1,ATT$(I,J)
				PRINT "ATT#"J;"->";ATT$(I,J)
			NEXT
	WEND
CLOSE #1
GOTO SEC4:

SDF:
rem   Extracts attributes from an SDF file
	L=1:I=0
	A$=BLOCK$(1)
	WHILE NOT EOF(1)
		IF I<>0 THEN INPUT #1,A$
		I=I+1
			BLOCK$(I)=MID$(A$,L,L(1)):L=L+L(1)
			X(I)=VAL(MID$(A$,L,L(2))):L=L+L(2)
			Y(I)=VAL(MID$(A$,L,L(3))):L=L+L(3)
			LAYER$(I)=MID$(A$,L,L(4)):L=L+L(4)
			FOR J=5 TO CNT+4
				ATT$(I,J-4)=MID$(A$,L,L(J))
				L=L+L(J)
			NEXT
		L=1
	WEND

SEC4:
NOIN=I
TOP=((CNT*36+(CNT-1)*24)/2)-36

REM -- SECTION FOUR - Create .DXF File -------------------------
PRINT
INPUT "File Name for .DXF file (no extension!!) : ";G$

REM ---- PART A -----
OPEN G$+".DXF" FOR OUTPUT AS #2
	PRINT #2,"  0" : PRINT #2,"SECTION"
	PRINT #2,"  2" : PRINT #2,"ENTITIES"
REM ---- PART B -----
	FOR I=1 TO NOIN
		PRINT #2,"  0" : PRINT #2,"INSERT"
		PRINT #2,"  8" : PRINT #2,NEWLAYER$
		PRINT #2," 66" : PRINT #2,"      1"
		PRINT #2,"  2" : PRINT #2,BLOCK$
		PRINT #2," 10" : PRINT #2,X(I)
		PRINT #2," 20" : PRINT #2,Y(I)
	IF Y$="n" OR Y$="N" THEN GOTO SKIP:
REM ---- PART C ----
		FOR J=1 TO CNT    'Number of attributes
			PRINT #2,"  0" : PRINT #2,"ATTRIB"
			PRINT #2,"  8" : PRINT #2,NEWLAYER$
			PRINT #2," 10" : PRINT #2,X(I)+216
			PRINT #2," 20" : PRINT #2,Y(I)+TOP-((j-1)*60)
			PRINT #2," 40" : PRINT #2,"36.0"
			PRINT #2,"  1" : PRINT #2,ATT$(I,J)
			PRINT #2,"  2" : PRINT #2,ATTTAG$(J)
			PRINT #2," 70" : PRINT #2,"     0"
		NEXT
REM ---- MORE PART B ----
	SKIP:
		PRINT #2,"  0" : PRINT #2,"SEQEND"
		PRINT #2,"  8" : PRINT #2,NEWLAYER$
	NEXT
REM ---- PART D ----
	PRINT #2,"  0" : PRINT #2,"ENDSEC"
	PRINT #2,"  0" : PRINT #2,"EOF"
CLOSE #2

REM -------------------------------------------------------------
