; ROT3D.LSP   by D.W. Nechodom, 2/23/88
;	rotates a set of line entities about one of the three axes (X,Y,Z)
;	and permits selection of an origin other than (0,0,0).
;
;	Z-axis rotation is included, but redundant; the AutoCAD "ROTATE"
;	command does a much better job.
;
;
(defun getdelta()
	(setq pt (polar origin (+ (angle origin p) rot)(distance origin p)))
	)
;
(defun entfix()
	(if (/= layer (assoc 8 e)) (setq e (subst layer (assoc 8 e) e)))
	(if (/= ltype nil) (setq e (append e (cons ltype nil))))
	(if (/= color nil) (setq e (append e (cons color nil))))
	(entmod e)
	)
;
(defun rotx()
	(setq p (list (cadr p1)(caddr p1)))
	(getdelta)
	(setq p1 (list (car p1)(car pt)(cadr pt)))
	(setq p (list (cadr p2)(caddr p2)))
	(getdelta)
	(setq p2 (list (car p2)(car pt)(cadr pt)))
	(if (/= p3 nil)(progn
		(setq p (list (cadr p3)(caddr p3)))
		(getdelta)
		(setq p3 (list (car p3)(car pt)(cadr pt)))
		))
	(if (/= p4 nil)(progn
		(setq p (list (cadr p4)(caddr p4)))
		(getdelta)
		(setq p4 (list (car p4)(car pt)(cadr pt)))
		))
	)
;
(defun roty()
	(setq p (list (car p1)(caddr p1)))
	(getdelta)
	(setq p1 (list (car pt)(cadr p1)(cadr pt)))
	(setq p (list (car p2)(caddr p2)))
	(getdelta)
	(setq p2 (list (car pt)(cadr p2)(cadr pt)))
	(if (/= p3 nil)(progn
		(setq p (list (car p3)(caddr p3)))
		(getdelta)
		(setq p3 (list (car pt)(cadr p3)(cadr pt)))
		))
	(if (/= p4 nil)(progn
		(setq p (list (car p4)(caddr p4)))
		(getdelta)
		(setq p4 (list (car pt)(cadr p4)(cadr pt)))
		))
	)
;
(defun rotz()
	(setq p (list (car p1)(cadr p1)))
	(getdelta)
	(setq p1 (list (car pt)(cadr pt)(caddr p1)))
	(setq p (list (car p2)(cadr p2)))
	(getdelta)
	(setq p2 (list (car pt)(cadr pt)(caddr p2)))
	(if (/= p3 nil)(progn
		(setq p (list (car p3)(cadr p3)))
		(getdelta)
		(setq p3 (list (car pt)(cadr pt)(caddr p3)))
		))
	(if (/= p4 nil)(progn
		(setq p (list (car p4)(cadr p4)))
		(getdelta)
		(setq p4 (list (car pt)(cadr pt)(caddr p4)))
		))
	)
;
(defun conv3dline()
	(setq pt1 (list (cadr (assoc 10 e))(caddr (assoc 10 e))))
	(setq pt2 (list (cadr (assoc 11 e))(caddr (assoc 11 e))))
	(setq layer (assoc 8 e) ltype (assoc 6 e) color (assoc 62 e))
	(entdel (cdar e))
	(command "LINE" pt1 pt2 "")
	(setq e (entget (entlast)))
	(entfix)
	(if (/= pt3 nil) (progn
		(command "LINE" pt2 pt3 pt4 pt1 "")
		(setq lcnt 3)
		(while (> lcnt 0)
			(setq e (entget (entnext (cdar e))))
			(entfix)
			(setq lcnt (1- lcnt))
			)
		))
	(setq pt3 nil pt4 nil)
	)
;
(defun conv3dface()
	(setq pt3 (list (cadr (assoc 12 e))(caddr (assoc 12 e))))
	(setq pt4 (list (cadr (assoc 13 e))(caddr (assoc 13 e))))
	(conv3dline)
	)
;
(defun convline()
	(if (null(assoc 38 e))
		(setq zpt 0.0)
		(setq zpt (cdr (assoc 38 e)))
		)
	(setq pt1 (append (cdr (assoc 10 e)) (cons zpt nil)))
	(setq pt2 (append (cdr (assoc 11 e)) (cons zpt nil)))
	(setq layer (assoc 8 e) ltype (assoc 6 e) color (assoc 62 e))
	(setq thk (assoc 39 e))
	(entdel (cdar e))
	(ssdel (cdar e) set)
	(setq item (1- item))
	(if (null thk)
		(command "3DLINE" pt1 pt2 "")
		(progn	(setq zoff (cdr (assoc 39 e)))
			(setq pt3 (list (car pt2)(cadr pt2) zoff))
			(setq pt4 (list (car pt1)(cadr pt1) zoff))
			(command "3DFACE" pt1 pt2 pt3 pt4 "")
			)
		)
	(setq e (entget (entlast)))
	(entfix)
	(ssadd (cdar e) set)
	)
;
(defun C:ROT3D()
	(setq blip (getvar "blipmode") echo (getvar "cmdecho"))
	(setvar "blipmode" 0)
	(setvar "cmdecho" 0)
	(setq test (list '"LINE" '"3DFACE" '"3DLINE"))
	(setq axis (getstring "\nRotate objects about axis (X,Y,Z): "))
	(setq axis (strcase axis))
	(setq org (getpoint "\n3D point of rotation axis : "))
	(if (or (null org)(null(listp org)))
		(setq origin (list '0.0 '0.0 (getvar "elevation")))
		(if (= 2 (length org))
			(setq origin (append org (cons (getvar "elevation") nil)))
			(setq origin org)
			)
		)
	(if (= axis "X")(setq origin (list(cadr origin)(caddr origin))))
	(if (= axis "Y")(setq origin (list(car origin)(caddr origin))))
	(if (= axis "Z")(setq origin (list(car origin)(cadr origin))))
	(setq rot (*(getreal "\nAngle of rotation : ")(/ pi 180.0)))
	(menucmd "S=X")(menucmd "S=SELECT")(terpri)
	(prompt "Use screen menu to pick objects ")(terpri)
	(setq set (ssget))
	(setq itemcount (sslength set))
	(setq item 0)
	(while (< item itemcount)
		(setq ent (ssname set item))
		(setq e (entget ent))
		(setq etype (cdr (assoc 0 e)))
		(setq qualify (member etype test))
		(if (null qualify) (progn
			(ssdel ent set)
			(setq itemcount (1- itemcount) item (1- item))
			))
		(if (= "LINE" (car qualify)) (convline))
		(setq item (1+ item))
		)
	(setq item 0)
	(while (< item itemcount)
		(setq e (entget(ssname set item)))
		(setq p1 (cdr(assoc 10 e)))
		(setq p2 (cdr(assoc 11 e)))
		(if (null(assoc 12 e))
			(setq p3 nil)
			(setq p3 (cdr(assoc 12 e))))
		(if (null(assoc 13 e))
			(setq p4 nil)
			(setq p4 (cdr(assoc 13 e))))
		(if (= axis "X") (rotx))
		(if (= axis "Y") (roty))
		(if (= axis "Z") (rotz))
		(setq item (1+ item))
		(setq e (subst (cons 10 p1)(assoc 10 e) e))
		(setq e (subst (cons 11 p2)(assoc 11 e) e))
		(if (null p3) ()
			(setq e (subst (cons 12 p3)(assoc 12 e) e)))
		(if (null p4) ()
			(setq e (subst (cons 13 p4)(assoc 13 e) e)))
		(entmod e)
		)
	(setvar "blipmode" blip)
	(setvar "cmdecho"echo)
	(redraw)
	(setq end "Done")
	)
;
(defun C:STRIPZ()
	(setq blip (getvar "blipmode") echo (getvar "cmdecho"))
	(setvar "blipmode" 0)
	(setvar "cmdecho" 0)
	(if (null end) (progn
		(menucmd "S=X")(menucmd "S=SELECT")(terpri)
		(prompt "Use screen menu to pick objects ")(terpri)
		(setq set (ssget))
		))
	(setq itemcount (sslength set))
	(setq item 0)
	(while (< item itemcount)
		(setq ent (ssname set item))
		(setq e (entget ent))
		(setq etype (cdr (assoc 0 e)))
		(if (= etype "3DLINE") (conv3dline))
		(if (= etype "3DFACE") (conv3dface))
		(setq item (1+ item))
		)
	(setvar "blipmode" blip)
	(setvar "cmdecho"echo)
	(redraw)
	(prompt "Done")(terpri)
	(setq exit nil)
	)
